use nettle_sys::{nettle_ecdsa_sign, nettle_ecdsa_verify};
use std::mem::zeroed;

use crate::{
    dsa,
    random::Random,
    ecc::{Point, Scalar},
};

/// Signs `digest` using the key `private`.
///
/// Returns the signature.
pub fn sign<R: Random>(
    private: &Scalar,
    digest: &[u8],
    random: &mut R,
) -> dsa::Signature {
    unsafe {
        let mut ret = zeroed();

        nettle_ecdsa_sign(
            &private.scalar,
            random.context(),
            Some(R::random_impl),
            digest.len(),
            digest.as_ptr(),
            &mut ret as *mut _,
        );
        dsa::Signature { signature: ret }
    }
}

/// Verify `signature` of `digest` using the key `public`.
///
/// Returns `true` if the signature is valid.
pub fn verify(
    public: &Point,
    digest: &[u8],
    signature: &dsa::Signature,
) -> bool {
    unsafe {
        nettle_ecdsa_verify(
            &public.point,
            digest.len(),
            digest.as_ptr(),
            &signature.signature as *const _,
        ) == 1
    }
}
